--[[
****************************************************************************************
Welfare Epics
$Date: 2008-05-30 17:23:25 -0400 (Fri, 30 May 2008) $
$Rev: 75573 $

Author: Ackis on Illidan US Horde
****************************************************************************************

Please see Wowace.com for more information.

****************************************************************************************
]]


local L			= LibStub("AceLocale-3.0"):GetLocale("WelfareEpics")

WelfareEpics	= LibStub("AceAddon-3.0"):NewAddon("WelfareEpics", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")

local addon = WelfareEpics

local GetMerchantItemLink = GetMerchantItemLink
local GetMerchantItemInfo = GetMerchantItemInfo

-- List of indexes of all items availible to the vendor
local vendoritems = nil


-- Returns configuration options for profiling
local function giveProfiles()

	local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)
	return profiles

end

-- Returns configuraion options for filter
local function giveFilter()

	local filter =
	{
		desc =
		{
			order = 1,
			type = "description",
			name = L["FILTER_OPTIONS"] .. "\n",
		},
		factions =
		{
			name	= L["Usable"],
			desc	= L["USABLE_TOGGLE"],
			type	= "toggle",
			get		= function() return addon.db.profile.usable end,
			set		= function() addon.db.profile.usable = not addon.db.profile.usable end,
			order	= 2,
		},
	}

end

-- Register slash commands and profile defaults
function addon:OnInitialize()

	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	self.db = LibStub("AceDB-3.0"):New("AckisRecipeListDB",defaults,"Default")

	-- Create the options with Ace3
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Welfare Epics",giveOptions,"Welfare Epics")
	AceConfigReg:RegisterOptionsTable("Welfare Epics Filter",giveFilter)
	AceConfigReg:RegisterOptionsTable("Welfare Epics Profile",giveProfiles)

	-- Add the options to blizzard frame (add them backwards so they show up in the proper order
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("Welfare Epics","Welfare Epics")
	self.optionsFrame[L["Profile"]] = AceConfigDialog:AddToBlizOptions("Welfare Epics Profile", L["Profile"], "Welfare Epics")
	self.optionsFrame[L["Filter"]] = AceConfigDialog:AddToBlizOptions("Welfare Epics Filter", L["Filter"], "Welfare Epics")

	-- Register slash commands
	self:RegisterChatCommand("welfareepics", "ChatCommand")

	-- Set default options, which are to include everything in the scan
	self.db:RegisterDefaults(
		{ profile =
			{
			-- Filter Options
		    usable = true,
			}
		}
	)

end

-- Register events and hooks
function addon:OnEnable()

	-- Register events
	self:RegisterEvent("MERCHANT_SHOW")
	self:RegisterEvent("MERCHANT_CLOSE")

	-- Hook into the merchant frame update running code after it is run
	self:RawHook("MerchantFrame_UpdateMerchantInfo", "HideUnusable")

end

-- Scans the vendor list adding item indexes to a local table
function addon:MERCHANT_SHOW()

	self:ParseItems()

end

-- Nils out the table
function addon:MERCHANT_CLOSE()

	vendoritems = nil

end

-- Scans a given item link outputting the type of armour/weapon it is, and what type of piece (melee, caster, healer, tank)
function addon:ScanLink(link)

	return "Cloth","Caster DPS"

end

-- Scans the availible items at a vendor
function addon:ParseItems()

	vendoritems = {}

	local numMerchantItems = GetMerchantNumItems()

	for i=1,numMerchantItems,1 do

		local _, _, _, _, _, isUsable = GetMerchantItemInfo(i)
		if isUsable then
			table.insert(vendoritems,i)
		end

	end

end

-- Same as blizzard code but references the table listing indexes of usable items only
function addon:HideUnusable()

	MerchantNameText:SetText(UnitName("NPC"))
	SetPortraitTexture(MerchantFramePortrait, "NPC")

	local numMerchantItems = GetMerchantNumItems()
 
	MerchantPageText:SetFormattedText(MERCHANT_PAGE_NUMBER, MerchantFrame.page, math.ceil(numMerchantItems / MERCHANT_ITEMS_PER_PAGE))

	local name, texture, price, quantity, numAvailable, isUsable, extendedCost

	for i=1, MERCHANT_ITEMS_PER_PAGE, 1 do

		local index = (((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE) + vendoritems[i])
		local itemButton = getglobal("MerchantItem"..i.."ItemButton")
		local merchantButton = getglobal("MerchantItem"..i)
		local merchantMoney = getglobal("MerchantItem"..i.."MoneyFrame")

		local merchantAltCurrency = getglobal("MerchantItem"..i.."AltCurrencyFrame")
		if ( index <= numMerchantItems ) then
			name, texture, price, quantity, numAvailable, isUsable, extendedCost = GetMerchantItemInfo(index)
			getglobal("MerchantItem"..i.."Name"):SetText(name)
			SetItemButtonCount(itemButton, quantity)
			SetItemButtonStock(itemButton, numAvailable)
			SetItemButtonTexture(itemButton, texture)
	 
				if ( extendedCost and (price <= 0) ) then
					itemButton.extendedCost = true
					itemButton.link = GetMerchantItemLink(index)
					itemButton.texture = texture
					MerchantFrame_UpdateAltCurrency(index, i)
					merchantAltCurrency:ClearAllPoints()
					merchantAltCurrency:SetPoint("BOTTOMLEFT", "MerchantItem"..i.."NameFrame", "BOTTOMLEFT", 0, 31)
					merchantMoney:Hide()
					merchantAltCurrency:Show()
				elseif ( extendedCost and (price > 0) ) then
					itemButton.extendedCost = true
					itemButton.link = GetMerchantItemLink(index)
					itemButton.texture = texture
					MerchantFrame_UpdateAltCurrency(index, i)
					MoneyFrame_Update(merchantMoney:GetName(), price)
					merchantAltCurrency:ClearAllPoints()
					merchantAltCurrency:SetPoint("LEFT", merchantMoney:GetName(), "RIGHT", -14, 0)
					merchantAltCurrency:Show()
					merchantMoney:Show()
				else
					itemButton.extendedCost = nil
					MoneyFrame_Update(merchantMoney:GetName(), price)
					merchantAltCurrency:Hide()
					merchantMoney:Show()
				end
	 
				itemButton.hasItem = true
				itemButton:SetID(index)
				itemButton:Show()
				if ( numAvailable == 0 ) then
					-- If not available and not usable
					if ( not isUsable ) then
						SetItemButtonNameFrameVertexColor(merchantButton, 0.5, 0, 0)
						SetItemButtonSlotVertexColor(merchantButton, 0.5, 0, 0)
						SetItemButtonTextureVertexColor(itemButton, 0.5, 0, 0)
						SetItemButtonNormalTextureVertexColor(itemButton, 0.5, 0, 0)
					else
						SetItemButtonNameFrameVertexColor(merchantButton, 0.5, 0.5, 0.5)
						SetItemButtonSlotVertexColor(merchantButton, 0.5, 0.5, 0.5)
						SetItemButtonTextureVertexColor(itemButton, 0.5, 0.5, 0.5)
						SetItemButtonNormalTextureVertexColor(itemButton,0.5, 0.5, 0.5)
					end
	 
				elseif ( not isUsable ) then
					SetItemButtonNameFrameVertexColor(merchantButton, 1.0, 0, 0)
					SetItemButtonSlotVertexColor(merchantButton, 1.0, 0, 0)
					SetItemButtonTextureVertexColor(itemButton, 0.9, 0, 0)
					SetItemButtonNormalTextureVertexColor(itemButton, 0.9, 0, 0)
				else
					SetItemButtonNameFrameVertexColor(merchantButton, 0.5, 0.5, 0.5)
					SetItemButtonSlotVertexColor(merchantButton, 1.0, 1.0, 1.0)
					SetItemButtonTextureVertexColor(itemButton, 1.0, 1.0, 1.0)
					SetItemButtonNormalTextureVertexColor(itemButton, 1.0, 1.0, 1.0)
				end
			else
				itemButton.hasItem = nil
				itemButton:Hide()
				SetItemButtonNameFrameVertexColor(merchantButton, 0.5, 0.5, 0.5)
				SetItemButtonSlotVertexColor(merchantButton,0.4, 0.4, 0.4)
				getglobal("MerchantItem"..i.."Name"):SetText("")
				getglobal("MerchantItem"..i.."MoneyFrame"):Hide()
				getglobal("MerchantItem"..i.."AltCurrencyFrame"):Hide()
			end
		end
 
		-- Handle repair items
		MerchantFrame_UpdateRepairButtons()
		 
		-- Handle vendor buy back item
		local buybackName, buybackTexture, buybackPrice, buybackQuantity, buybackNumAvailable, buybackIsUsable = GetBuybackItemInfo(GetNumBuybackItems())
		if ( buybackName ) then
			MerchantBuyBackItemName:SetText(buybackName)
			SetItemButtonCount(MerchantBuyBackItemItemButton, buybackQuantity)
			SetItemButtonStock(MerchantBuyBackItemItemButton, buybackNumAvailable)
			SetItemButtonTexture(MerchantBuyBackItemItemButton, buybackTexture)
			MerchantBuyBackItemMoneyFrame:Show()
			MoneyFrame_Update("MerchantBuyBackItemMoneyFrame", buybackPrice)
			MerchantBuyBackItem:Show()
		 
		else
			MerchantBuyBackItemName:SetText("")
			MerchantBuyBackItemMoneyFrame:Hide()
			SetItemButtonTexture(MerchantBuyBackItemItemButton, "")
			SetItemButtonCount(MerchantBuyBackItemItemButton, 0)
		-- Hide the tooltip upon sale
			if ( GameTooltip:IsOwned(MerchantBuyBackItemItemButton) ) then
				GameTooltip:Hide()
			end
		end
		 
		-- Handle paging buttons
		if ( numMerchantItems > MERCHANT_ITEMS_PER_PAGE ) then
			if ( MerchantFrame.page == 1 ) then
				MerchantPrevPageButton:Disable()
			else
				MerchantPrevPageButton:Enable()
			end
			if ( MerchantFrame.page == ceil(numMerchantItems / MERCHANT_ITEMS_PER_PAGE) or numMerchantItems == 0) then
				MerchantNextPageButton:Disable()
			else
				MerchantNextPageButton:Enable()
			end
			MerchantPageText:Show()
			MerchantPrevPageButton:Show()
			MerchantNextPageButton:Show()
		else
			MerchantPageText:Hide()
			MerchantPrevPageButton:Hide()
			MerchantNextPageButton:Hide()
		end
		 
		-- Show all merchant related items
		MerchantBuyBackItem:Show()
		MerchantFrameBottomLeftBorder:Show()
		MerchantFrameBottomRightBorder:Show()
		 
		-- Hide buyback related items
		MerchantItem11:Hide()
		MerchantItem12:Hide()
		BuybackFrameTopLeft:Hide()
		BuybackFrameTopRight:Hide()
		BuybackFrameBotLeft:Hide()
		BuybackFrameBotRight:Hide()
		 
		-- Position merchant items
		MerchantItem3:SetPoint("TOPLEFT", "MerchantItem1", "BOTTOMLEFT", 0, -8)
		MerchantItem5:SetPoint("TOPLEFT", "MerchantItem3", "BOTTOMLEFT", 0, -8)
		MerchantItem7:SetPoint("TOPLEFT", "MerchantItem5", "BOTTOMLEFT", 0, -8)
		MerchantItem9:SetPoint("TOPLEFT", "MerchantItem7", "BOTTOMLEFT", 0, -8)
	end


do

	local merchantlist = {}
	--local merchantlist["Cloth"] = {}
	--local merchantlist["Leather"] = {}
	--local merchantlist["Mail"] = {}
	--local merchantlist["Plate"] = {}
	--local merchantlist["Weapon"] = {}
	--local merchantlist["Other"] = {}

	function addon:GetVendorItems()

		local numMerchantItems = GetMerchantNumItems()
		-- THings I know:
		-- MerchantFrame.page = page displayed
		-- index = all the items on the merchant
		-- MERCHANT_ITEMS_PER_PAGE max items per page

		-- Reset the armour type table
		for k in pairs(merchantlist["Cloth"]) do merchantlist["Cloth"][k] = nil end
		for k in pairs(merchantlist["Leahter"]) do merchantlist["Leather"][k] = nil end
		for k in pairs(merchantlist["Mail"]) do merchantlist["Mail"][k] = nil end
		for k in pairs(merchantlist["Plate"]) do merchantlist["Plate"][k] = nil end
		-- Reset the weapon type table
		for k in pairs(merchantlist["Weapon"]) do merchantlist["Weapon"][k] = nil end
		-- Reset the other type table
		for k in pairs(merchantlist["Other"]) do merchantlist["Other"][k] = nil end

		-- Build list of sorted items from the merchant
		for i=1,numMerchantItems,1 do
			local itemtype,classtype = addon:ScanLink(GetMerchantItemLink(i))
			local _, _, _, _, _, isUsable = GetMerchantItemInfo(i)
			merchantlist[itemtype][i] = isUsable
		end

	end

end

-- Adds new tabs to the vendor windows for specific types of items
function addon:SetupNewTabs()

	addon.NewTab1 = CreateFrame("Button","MerchantFrameTab3",MerchantFrameTab3,"CharacterFrameTabButtonTemplate")
	addon.NewTab1:SetText("Cloth")
	addon.NewTab1:ClearAllPoints()
	addon.NewTab1:SetPoint("LEFT",MerchantFrameTab2,"RIGHT",-16,0)
	addon.NewTab1:SetScript("OnEnter", function(this)
										GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
										GameTooltip:SetText("", 1.0,1.0,1.0)
									end)
	addon.NewTab1:SetScript("OnLeave", function(this)
										GameTooltip:Hide()
									end)
	addon.NewTab1:SetScript("OnEnter", function(this)
										PanelTemplates_SetTab(MerchantFrame, this:GetID())
										MerchantFrame_Update()
									end)

end

